unit acyFile;
{    This file is part of the AVR Bootloader by Arne Krüger for the AVRootloader by Hagen Re.

    AVR Bootloader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
	any later version.

    AVR Bootloader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AVR Bootloader.  If not, see <http://www.gnu.org/licenses/>.
	
	This file is programmed by Arne Krüger news.kruegerarne@web.de
}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,globalfunctions,Dialogs;

type
  { THexFile }

  { TAcyFile }

  TAcyFile = class
    public
      constructor Create;
      procedure LoadFromFile(path:string);
      function GetLatestBufferSize:integer;
      function GetTotalTranferedBytesTotal:integer;

      function GetCatchString:string;
      function GetStartAddress:integer;
      function ReadContinue(var Buf:array of byte;Length:integer):integer;
    private
      LatestBufferSize:integer;
      TransferedBytesTotal:integer;
      BlockSize:integer;
      Address:integer;
      AcyFile:TFileStream;
   end;

implementation

{ TAcyFile }

constructor TAcyFile.Create;
begin
  inherited create;
  AcyFile := nil;
  TransferedBytesTotal := 0;
  LatestBufferSize := 0;
  BlockSize := 0;
end;

procedure TAcyFile.LoadFromFile(path: string);
var
  buf:array[0..1023] of byte;
  l:integer;
begin
  if AcyFile <> nil then
    AcyFile.Free;
  AcyFile := TFileStream.Create(path,fmOpenRead);
  BlockSize := 0;
  TransferedBytesTotal := 0;
  repeat                       //lets look for the setAddress comand for know where we will write our file to
    l := ReadContinue(Buf, high(buf)+1);
  until (l = 0) or ((buf[0] = $FF) and (l = 4));
  Address := buf[1] shl 16;
  Address := Address + buf[2] shl 8;
  Address := Address + buf[3];
  AcyFile.Position:=0;
  BlockSize := 0;
  TransferedBytesTotal := 0;
end;




function TAcyFile.GetLatestBufferSize: integer;
begin
  result := LatestBufferSize;
end;

function TAcyFile.GetTotalTranferedBytesTotal: integer;
begin
  result := TransferedBytesTotal;
end;

function TAcyFile.GetCatchString: string;
var
  posi:int64;
  i:integer;
  b:byte;
begin
  posi := AcyFile.Position;
  AcyFile.Position:=0;
  result := '';
  repeat
    b := AcyFile.ReadByte;
    if IsASCII(b) then
      result := result + chr(b);
  until (AcyFile.Size = AcyFile.Position) or IsASCII(b);

  AcyFile.Position:=posi;
end;

function TAcyFile.GetStartAddress:integer;
begin
  result := Address;
end;

function TAcyFile.ReadContinue(var Buf: array of byte; Length: integer): integer;
var
  b1,b2:byte;
  found:boolean;
  i,l:integer;
  p:Int64;
begin
  result := 0;
  try
    if AcyFile.Position< AcyFile.Size then begin
    if (BlockSize=0) then begin
      found := false;
      if (AcyFile.Position=0) then begin
        b2 := 0;
        repeat
          b1 := AcyFile.ReadByte;
          if (b1=$FE) and (b2=$FE) then
            found := true;
          b2 := b1;
        until (AcyFile.Size = AcyFile.Position) or found;
         p :=  AcyFile.Position -2;
         AcyFile.Position :=  p;
      end else begin
        found := true;
      end;
      if found then begin
        Buf[0] := AcyFile.ReadByte;
        if (Buf[0] = $01) or (Buf[0] = $02) then     //Write Flash or Erase Flash Command?
          l := 2
        else
          l := 4;
        if l > Length then begin
          result := 0;
        end else begin
          for i := 1 to l-1 do begin
            b1 := AcyFile.ReadByte;
            Buf[i] :=b1;
          end;
          result := l;
        end;
      end;
      if Buf[0] = $FE then begin    //Write Buffer Command(which should have the size of the next buffer)?
        BlockSize:= Buf[2] shl 8;
        BlockSize:= BlockSize+Buf[3];
      end else
        BlockSize := 0;
      LatestBufferSize:= result;
    end else begin
      if BlockSize > Length then begin
        result := 0;
      end else begin
        for i := 0 to BlockSize-1 do begin
          if (AcyFile.Size = AcyFile.Position) then begin
            ShowMessage('EOF i='+inttostr(i)+' BlockSize-1='+inttostr(BlockSize-1)+' pos='+inttostr(AcyFile.Position));
            break;
          end else
            Buf[i] := AcyFile.ReadByte;
        end;
        result := BlockSize;
        TransferedBytesTotal := TransferedBytesTotal + result;
        LatestBufferSize:= result;
      end;
      BlockSize:=0;
    end;
    end;
  except on EReadError do begin
          end;
  end;
end;



end.

