program bootlaodergui;
{    This file is part of the AVR Bootloader by Arne Krüger for the AVRootloader by Hagen Re.

    AVR Bootloader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
	any later version.

    AVR Bootloader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AVR Bootloader.  If not, see <http://www.gnu.org/licenses/>.
	
	This file is programmed by Arne Krüger news.kruegerarne@web.de
}
{$mode objfpc}{$H+}

uses
  {$IFDEF LINUX}
    {$DEFINE UseCThreads}
  {$ENDIF}
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  Interfaces, // this includes the LCL widgetset
  Forms
  { you can add units after this }, unit1,SysUtils, synafpc, synautil, synaser,
bootloader, epiktimer, etpackage, HexFile, acyFile, globalfunctions, inputFile;
type

{ TBootInterface }

TBootInterface=class
  public
    constructor create;
    destructor destroy; override;
    procedure ProgramFile(Path:string);
    procedure SetTTY(Path:string;baud:integer);
    function GetFinished:boolean;
    function GetError:boolean;
  private
    BootLoader:TBootloader;
    LastString:string;
    started:boolean;
    ErrorOccured:boolean;
    procedure OnBootStatus(StatusMsg: string;Status: TBootloaderStatus; const ApplyNewLine:boolean);
    procedure OnBootError(StatusMsg: string; Error: TBootloaderError);
    procedure OnBootInfo(BootInfo: TBootInfo);
end;

{ TBootInterface }

constructor TBootInterface.create;
begin
  inherited create;
  started := false;
  LastString:='';
  BootLoader := TBootloader.create;
  BootLoader.OnStatus:=@OnBootStatus;
  BootLoader.OnBootInfo := @OnBootInfo;
  BootLoader.OnError:= @OnBootError;
end;

destructor TBootInterface.destroy;
begin
  BootLoader.Free;
  inherited destroy;
end;

procedure TBootInterface.ProgramFile(Path: string);
begin
  BootLoader.ProgramFlash(Path);
  started := false;
  ErrorOccured := false;
end;

procedure TBootInterface.SetTTY(Path: string; baud: integer);
begin
  BootLoader.SetTTY(path);
  BootLoader.SetBaud(baud);
end;

function TBootInterface.GetFinished: boolean;
begin
  result :=  started and (BootLoader.GetStatus=bsIDLE);
end;

function TBootInterface.GetError: boolean;
begin
  result := ErrorOccured;
end;

procedure TBootInterface.OnBootStatus(StatusMsg: string;
  Status: TBootloaderStatus; const ApplyNewLine:boolean);
begin
  if Status= bsConnected then
    started := true;
  if StatusMsg <> '' then begin
    LastString:=StatusMsg;
    if ApplyNewLine then
      StatusMsg := StatusMsg+#$0D+#$0A;
    Write(StatusMsg);
  end;
end;

procedure TBootInterface.OnBootError(StatusMsg: string; Error: TBootloaderError);
begin
  if not ErrorOccured then
    ErrorOccured := (Error <> beSuccessPlainStyle) and (Error <> beSuccessCryptStyle);
  if length(LastString) < 19 then begin
    WriteLn(#9+#9+#9+#9+StatusMsg);
  end else if length(LastString) < 28 then begin
    WriteLn(#9+#9+#9+#9+StatusMsg);
  end else if length(LastString) < 36 then begin
    WriteLn(#9+#9+StatusMsg);
  end else begin
   WriteLn(#9+#9+#9+StatusMsg);
  end;
end;

procedure TBootInterface.OnBootInfo(BootInfo: TBootInfo);
begin
  writeln('Found AVRootloader (C) by Hagen Re with following data:');
  writeln(#9+'Hello: '+#9+#9+#9+BootInfo.Hello);
  writeln(#9+'AVR-Signatur: '+#9+#9+inttohex(BootInfo.Sign0,2)+' '+inttohex(BootInfo.Sign1,2)+' '+inttohex(BootInfo.Sign2,2));
  writeln(#9+'Version: '+#9+#9+inttostr(BootInfo.version));
  writeln(#9+'BootPages: '+#9+#9+inttostr(BootInfo.Bootpages));
  if BootInfo.Crypt then begin
    writeln(#9+'Expect crypted file:'+#9+'yes');
  end else begin
    writeln(#9+'Expect crypted file:'+#9+'no');
  end;
  writeln('');
end;

function ParseParamerter(s:string):string;
begin
  delete(s,1,pos('=',s));
  result := s;
end;

var
  i:integer;
  startwithgui:boolean;
  BootInterface:TBootInterface;
  fn:string;
  bd:integer;
  tty:string;
begin
  startwithgui:=true;
  for i := 0 to Paramcount do begin
    if Lowercase(ParamStr(i)) = '-nogui' then begin
      startwithgui := false;
    end else if pos('-tty',Lowercase(ParamStr(i))) > 0 then begin
      tty := ParseParamerter(ParamStr(i));
    end else if pos('-bd',Lowercase(ParamStr(i))) > 0 then begin
      bd := StrToInt(ParseParamerter(ParamStr(i)));
    end else if pos('-file',Lowercase(ParamStr(i))) > 0 then begin
      fn := ParseParamerter(ParamStr(i));
    end;
  end;
  if startwithgui then begin
     Application.Initialize;
    Application.CreateForm(TForm1, Form1);
    Application.Run;
  end else begin
    writeln('');
    writeln('AVR Bootloader UI by Arne Krüger');
    BootInterface := TBootInterface.create;
    try
      writeln('');
      writeln('TTY: '+tty);
      writeln('fn: '+fn);
      writeln('Baud: '+inttostr(bd));
      writeln('');
      if (IsTTYOK(tty)) and (bd <> 0) and (FileExists(fn)) then begin
        BootInterface.SetTTY(tty,bd);
        BootInterface.ProgramFile(fn);
        repeat
          Application.ProcessMessages;
        until BootInterface.getfinished or BootInterface.GetError;
      end else begin
        Writeln('Wrong Parameters!');
      end;
      writeln('');
      Write('Finished ');
      if BootInterface.GetError then
        writeln('with errors!')
      else
        WriteLn('');
    finally
      BootInterface.Free;
    end;
  end;
end.

