unit HexFile;
{    This file is part of the AVR Bootloader by Arne Krüger for the AVRootloader by Hagen Re.

    AVR Bootloader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
	any later version.

    AVR Bootloader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AVR Bootloader.  If not, see <http://www.gnu.org/licenses/>.
	
	This file is programmed by Arne Krüger news.kruegerarne@web.de
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;
type
  TRecType=(rcData=0,rcEOF=1,rcExtendedSegmentAddressRecord=2,rcStartSegmentAddressRecord=3,rcExtendedLinearAddressRecord=4,rcStartLinearAddressRecord=5);

  THexFilePointer = record
    RecIndex : integer;
    ByteIndex : integer;
    Address:integer;
  end;

  THexExtendedAdr = record
    ExtendedAddress : integer;
    RecIndex : integer;
  end;
  { THexFileRecord }

  THexFileRecord = class
    public
      constructor Create(rec:string);
      procedure LoadRec(rec:string);
      procedure LoadRecWithoutDataAndCheck(rec:string);
      Address:integer;
      RecLength:integer;
      Buffer:array[0..255] of Byte;
      valid:boolean;
      Typ:TRecType;
    private

  end;

  { THexFile }
  THexFile = class
    public
      constructor Create;
      procedure LoadFromFile(path:string);
      function GotoAddress(Address:integer):integer;
      function GetNextAddress:integer;
      function GetStartAddress:integer;  //Here we get the start address about the block, which had been transfered lately
      function GetLatestBufferSize:integer;
      function GetTotalTranferedBytesTotal:integer;
      function ReadBuffer(address:integer;var Buf:array of byte;Length:integer):integer;
      function ReadContinue(var Buf:array of byte;Length:integer):integer;
      procedure SetPadding(PageSize:integer;Padding:Byte);
    private
      LatestBufferStart:integer;
      LatestBufferSize:integer;
      TransferedBytesTotal:integer;

      HFile:TStringList;
      PaddingPageSize:integer;
      PaddingByte:Byte;
      ActualAddress:THexFilePointer;
      ExtendedAddresses:TStringlist;  //odd index points to index of HexFile, even keeps Address
      FirstReclength:integer;
  end;
implementation

function LookForExtendedAdr(Adr:integer;lst:TStringList):THexExtendedAdr;
var
  i:integer;
  exttmp:integer;
begin
  Result.RecIndex:=0;
  Result.ExtendedAddress:=0;
  for i := 0 to (lst.Count div 2)-1 do begin
    exttmp:= StrToInt(lst[(i*2)+1]);
    if exttmp < Adr then begin
      Result.ExtendedAddress:= exttmp;
      result.RecIndex:= StrToInt(lst[i*2]);
    end else
      break;
  end;
end;

{ THexFileRecord }

constructor THexFileRecord.Create(rec: string);
begin
  inherited Create;
  if rec <> '' then
    LoadRec(rec);
end;

procedure THexFileRecord.LoadRecWithoutDataAndCheck(rec: string);
var
  i,n:integer;
  tmp:string;
  check:byte;
  b:byte;
begin
  if length(rec) >8 then begin
    delete(rec,1,pos(':',rec));
    tmp := rec[1]+rec[2];
    RecLength:=strToInt('$'+tmp);
    tmp:=rec[3]+rec[4]+rec[5]+rec[6];
    Address:=strToInt('$'+tmp);
    tmp:=rec[7]+rec[8];
    typ:=TRecType(strToInt('$'+tmp));
    valid := true;
  end else
    valid := false;
end;

procedure THexFileRecord.LoadRec(rec: string);
var
  i,n:integer;
  tmp:string;
  check:byte;
  b:byte;
begin
  if length(rec) >8 then begin
    delete(rec,1,pos(':',rec));
    tmp := rec[1]+rec[2];
    RecLength:=strToInt('$'+tmp);
    check := RecLength;
    tmp:=rec[3]+rec[4]+rec[5]+rec[6];
    Address:=strToInt('$'+tmp);
    check := check + Address+(Address shr 8);
    tmp:=rec[7]+rec[8];
    typ:=TRecType(strToInt('$'+tmp));
    check := check + ord(typ);
    n := 9;
    for i := 1 to RecLength do begin
      tmp := rec[n]+rec[n+1];
      n := n+2;
      b := strToInt('$'+tmp);
      check := check + b;
      Buffer[i-1]:=b;
    end;
    tmp := rec[n]+rec[n+1];
    check := check +strToInt('$'+tmp);
    valid := check = 0;
  end else
    valid := false;
end;

{ THexFile }

constructor THexFile.Create;
begin
  inherited;
  HFile := TStringList.Create;
  ExtendedAddresses:=TStringList.Create;
  ActualAddress.ByteIndex := 0;
  ActualAddress.RecIndex:= 0;
  ActualAddress.Address:=0;
  LatestBufferStart := 0;
  TransferedBytesTotal := 0;
  PaddingPageSize := 1;
  PaddingByte:= $FF;
end;

procedure THexFile.LoadFromFile(path: string);
var
  hr:THexFileRecord;
  i:integer;
  ExtAdr:longint;
begin
  ExtendedAddresses.clear;
  HFile.LoadFromFile(path);
  ActualAddress.Address := 0;
  ActualAddress.RecIndex:=0;
  ActualAddress.ByteIndex:=0;
  LatestBufferStart := 0;
  FirstReclength := 0;
  TransferedBytesTotal := 0;
  hr := THexFileRecord.Create('');
  try
    for i := 0 to HFile.count-1 do begin
      hr.LoadRecWithoutDataAndCheck(HFile[i]);
      if (hr.Typ=rcData) and (FirstReclength=0) then begin
        FirstReclength := hr.RecLength;
        ActualAddress.Address := hr.Address;
        ActualAddress.RecIndex:=i;
        ActualAddress.ByteIndex:=0;
        LatestBufferStart := hr.Address;
      end;
      if hr.Typ = rcExtendedSegmentAddressRecord then begin
        hr.LoadRec(HFile[i]);
        ExtAdr:=hr.Buffer[0];
        ExtAdr:=ExtAdr+ hr.Buffer[1] shl 8;
        ExtAdr:=ExtAdr shl 4;
        ExtendedAddresses.Add(inttostr(i));
        ExtendedAddresses.Add(Inttostr(ExtAdr));
      end else if hr.Typ = rcExtendedLinearAddressRecord then begin
        hr.LoadRec(HFile[i]);
        ExtAdr:=hr.Buffer[0];
        ExtAdr:=ExtAdr+ hr.Buffer[1] shl 8;
        ExtAdr:=ExtAdr shl 16;
        ExtendedAddresses.Add(inttostr(i));
        ExtendedAddresses.Add(Inttostr(ExtAdr));
      end;
    end;
  finally
    hr.free;
  end;
end;

function THexFile.GotoAddress(Address: integer): integer;
var
  i:integer;                     //To be tested with Extended Addresses!!!!
  Exttmp:LongInt;
  Addresstmp:integer;
  hr:THexFileRecord;
  ExtAdr:THexExtendedAdr;
begin
  ExtAdr := LookForExtendedAdr(Address,ExtendedAddresses);
  Addresstmp:= Address-extAdr.ExtendedAddress;
  if FirstReclength = 0 then FirstReclength := 1;
  i := Addresstmp div FirstReclength;
  i := i + ExtAdr.RecIndex;
  hr := THexFileRecord.Create('');
  try
    repeat
      hr.LoadRecWithoutDataAndCheck(HFile[i]);
      ExtAdr := LookForExtendedAdr(hr.Address,ExtendedAddresses);
      Addresstmp:= hr.Address+ExtAdr.ExtendedAddress;
      if (Addresstmp > Address) then begin
        dec(i);
      end else if (Addresstmp + hr.RecLength < Address) then begin
        inc(i);
      end else begin    //(hr.Address <= Address) and (hr.Address + hr.Length >= Address)
        ActualAddress.ByteIndex := Address - Addresstmp;                     //we found it
        ActualAddress.RecIndex:= i;
        ActualAddress.Address:=Address;
        break;
      end;
    until (i < 0) or (i > HFile.Count-1);
  finally
    hr.free;
  end;
  result := ActualAddress.Address;
end;

function THexFile.GetNextAddress: integer;
begin
  result := ActualAddress.Address;
end;

function THexFile.GetStartAddress: integer;
begin
  result := LatestBufferStart;
end;

function THexFile.GetLatestBufferSize: integer;
begin
  result := LatestBufferSize;
end;

function THexFile.GetTotalTranferedBytesTotal: integer;
begin
  result := TransferedBytesTotal;
end;

function THexFile.ReadBuffer(address: integer; var Buf: array of byte;
  Length: integer): integer;
begin
  if ActualAddress.Address <> address then begin
    GotoAddress(address);
  end;
  if ActualAddress.Address = address then begin
    result := ReadContinue(Buf,Length);
  end;
end;

function THexFile.ReadContinue(var Buf: array of byte; Length: integer): integer;
var
  i,n,m:integer;                     //To be tested with Extended Addresses!!!!
  hr:THexFileRecord;
  rdy:boolean;
  b:byte;
  t_:TRecType;
begin
  i := 0;
  rdy:=false;
  LatestBufferStart := ActualAddress.Address;
  hr := THexFileRecord.Create('');
  try
    while (not rdy) and (ActualAddress.RecIndex < HFile.count) and (hr.Typ<>rcEOF) do begin
      hr.LoadRec(HFile[ActualAddress.RecIndex]);
      if hr.Typ = rcData then begin
        for m := ActualAddress.ByteIndex to hr.RecLength-1 do begin
          if i > Length-1 then begin
            rdy := true;
            break;
          end;
          b := hr.Buffer[m];
          Buf[i] := b;
          inc(i);
          inc(ActualAddress.Address);
          inc(ActualAddress.ByteIndex);
        end;
        if (ActualAddress.ByteIndex>hr.RecLength-1) then begin
          ActualAddress.ByteIndex := 0;
          inc(ActualAddress.RecIndex);
        end;
      end else
        inc(ActualAddress.RecIndex);
    end;
    t_ := hr.Typ;
    m := HFile.Count-1;
    if (ActualAddress.RecIndex >= m) or (t_=rcEOF) then begin   //We couldnt read all bytes because of EOF
      while (ActualAddress.Address mod PaddingPageSize) <> 0 do begin           //so, lets see if we should use padding bytes
        Buf[i] := PaddingByte;
        inc(i);
        inc(ActualAddress.Address);
        inc(ActualAddress.ByteIndex);
        if i > Length-1 then begin
          //rdy := true;
          break;
        end;
      end;
    end;
    result := i;
    LatestBufferSize:= i;
    TransferedBytesTotal := TransferedBytesTotal+i;
  finally
     hr.free;
  end;
end;

procedure THexFile.SetPadding(PageSize: integer; Padding: Byte);
begin
  Self.PaddingByte:= Padding;
  Self.PaddingPageSize:=PageSize;
end;



end.

