unit inputFile;
{    This file is part of the AVR Bootloader by Arne Krüger for the AVRootloader by Hagen Re.

    AVR Bootloader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
	any later version.

    AVR Bootloader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AVR Bootloader.  If not, see <http://www.gnu.org/licenses/>.
	
	This file is programmed by Arne Krüger news.kruegerarne@web.de
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,acyfile,hexfile;

type
  TFileType = (ftNone,ftHex,ftAcy);
  { TAcyFile }

  { TInputFile }

  TInputFile = class
    public
      constructor create;
      destructor destroy; override;
      procedure LoadFromFile(path:string);
      function GetLatestBufferSize:integer;
      function GetTotalTranferedBytesTotal:integer;
      function GetNextAddress:integer;
      function GetStartAddress:integer;
      function ReadContinue(var Buf:array of byte;Length:integer):integer;
      procedure SetPadding(PageSize:integer;Padding:Byte);

      function GetFileType:TFileType;
    private
      acy:TAcyFile;
      hex:THexFile;
      ftype:TFileType;
    end;
implementation



{ TInputFile }

constructor TInputFile.create;
begin
  inherited create;
  acy := TAcyFile.create;
  hex := THexFile.create;
  ftype := ftNone
end;

destructor TInputFile.destroy;
begin
  acy.free;
  hex.free;
  inherited destroy;
end;

procedure TInputFile.LoadFromFile(path: string);
begin
  if lowercase(ExtractFileExt(path)) = '.acy' then begin
    ftype:= ftAcy;
    acy.LoadFromFile(path);
  end else if lowercase(ExtractFileExt(path)) = '.hex' then begin
    ftype:= ftHex;
    hex.LoadFromFile(path);
  end;
end;

function TInputFile.GetLatestBufferSize: integer;
begin
  case ftype of
    ftHex:  begin
              result := hex.GetLatestBufferSize;
            end;
    ftAcy:  begin
              result := acy.GetLatestBufferSize;
            end;
    else    result := 0;
  end;
end;

function TInputFile.GetTotalTranferedBytesTotal: integer;
begin
  case ftype of
    ftHex:  begin
              result := hex.GetTotalTranferedBytesTotal;
            end;
    ftAcy:  begin
              result := acy.GetTotalTranferedBytesTotal;
            end;
    else    result := 0;
  end;
end;

function TInputFile.GetNextAddress: integer;
begin
  result := 0;
  if ftype=ftHex then
    result := hex.GetNextAddress;
end;

function TInputFile.GetStartAddress: integer;
begin
  case ftype of
    ftHex:  begin
              result := hex.GetStartAddress;
            end;
    ftAcy:  begin
              result := acy.GetStartAddress;
            end;
    else    result := 0;
  end;
end;

function TInputFile.ReadContinue(var Buf: array of byte; Length: integer
  ): integer;
begin
  case ftype of
    ftHex:  begin
              result := hex.ReadContinue(Buf,Length);
            end;
    ftAcy:  begin
              result := acy.ReadContinue(Buf,Length);
            end;
    else    result := 0;
  end;
end;

procedure TInputFile.SetPadding(PageSize: integer; Padding: Byte);
begin
  if ftype=ftHex then
    hex.SetPadding(PageSize,Padding);
end;

function TInputFile.GetFileType: TFileType;
begin
  result := ftype;
end;

end.

