unit unit1;
{    This file is part of the AVR Bootloader by Arne Krüger for the AVRootloader by Hagen Re.

    AVR Bootloader is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
	any later version.

    AVR Bootloader is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with AVR Bootloader.  If not, see <http://www.gnu.org/licenses/>.
	
	This file is programmed by Arne Krüger news.kruegerarne@web.de
}
{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls,synaser, process, ExtCtrls, bootloader,Inifiles,globalfunctions;

type

  { TForm1 }

  TForm1 = class(TForm)
    btnConnect: TButton;
    btnProgram: TButton;
    btnDisconnect: TButton;
    chbShowRx: TCheckBox;
    cmbFiles: TComboBox;
    cmbInterface: TComboBox;
    cmbBaud: TComboBox;
    edtRxWin: TEdit;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    lblCrypt: TLabel;
    lblSgn: TLabel;
    lblHello: TLabel;
    lblVersion: TLabel;
    lblBootPages: TLabel;
    Memo1: TMemo;
    OpenDialog1: TOpenDialog;

    procedure btnProgramClick(Sender: TObject);
    procedure btnConnectClick(Sender: TObject);
    procedure btnDisconnectClick(Sender: TObject);
    procedure chbShowRxChange(Sender: TObject);
    procedure cmbBaudChange(Sender: TObject);
    procedure cmbFilesChange(Sender: TObject);
    procedure cmbInterfaceChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure OnBootStatus(StatusMsg: string;Status: TBootloaderStatus; const ApplyNewLine:boolean);
    procedure OnBootloaderRXByte(RxByte: Byte);
    procedure OnBootError(StatusMsg: string; Error: TBootloaderError);
    procedure OnBootInfo(BootInfo: TBootInfo);
  private
    BootLoader:TBootLoader;
    Configfile: TIniFile;
    FileList:TstringList;
    procedure OpenSerial;
    function CheckSerial:boolean;
    procedure SetButtons(enabled_:boolean);
    { private declarations }
  public
    { public declarations }
  end; 

  function IsTTYOK(path:string):boolean;
var
  Form1: TForm1;

implementation

function IsTTYOK(path: string): boolean;
begin
  {$IFDEF UNIX}
    result:= FileExists(path);
  {$ELSE}
    result := pos('COM',path)>1;
  {$ENDIF}
end;

{ TForm1 }

procedure TForm1.btnConnectClick(Sender: TObject);
begin
  OpenSerial;
  BootLoader.Connect;
end;

procedure TForm1.btnProgramClick(Sender: TObject);
var
  b:boolean;
begin
  b := true;
  if (cmbFiles.ItemIndex > -1) and (cmbFiles.ItemIndex < FileList.Count)  then begin
    if FileExists(FileList[cmbFiles.ItemIndex]) then begin
      OpenSerial;
      BootLoader.ProgramFlash(FileList[cmbFiles.ItemIndex]);
      b := false;
    end;
  end;
  if b then begin
    if OpenDialog1.Execute then begin
      FileList.add(OpenDialog1.FileName);
      cmbFiles.Items.Insert(cmbFiles.Items.Count-1,ExtractFileName(OpenDialog1.FileName));
      cmbFiles.ItemIndex:=cmbFiles.Items.Count-2;
      OpenSerial;
      BootLoader.ProgramFlash(OpenDialog1.FileName)
    end;
  end;
end;

procedure TForm1.btnDisconnectClick(Sender: TObject);
begin
  BootLoader.Disconnect;
end;

procedure TForm1.chbShowRxChange(Sender: TObject);
begin
  edtRxWin.Visible:= chbShowRx.Checked;
end;

procedure TForm1.cmbBaudChange(Sender: TObject);
begin
  btnDisconnect.Click;
end;

procedure TForm1.cmbFilesChange(Sender: TObject);
begin
  if cmbFiles.ItemIndex = cmbFiles.Items.Count-1 then begin
    if btnProgram.Enabled then begin
      btnProgram.Click;
    end else begin
      if OpenDialog1.Execute then begin
        FileList.add(OpenDialog1.FileName);
        cmbFiles.Items.Insert(cmbFiles.Items.Count-1,ExtractFileName(OpenDialog1.FileName));
        cmbFiles.ItemIndex:=cmbFiles.Items.Count-2;
      end;
    end;
  end;
end;



procedure TForm1.OnBootInfo(BootInfo: TBootInfo);
begin
  lblHello.Caption      := 'Hello: '+BootInfo.Hello;
  lblSgn.Caption        := 'AVR-Signatur: '+inttohex(BootInfo.Sign0,2)+' '+inttohex(BootInfo.Sign1,2)+' '+inttohex(BootInfo.Sign2,2);
  lblVersion.Caption    := 'Version: '+inttostr(BootInfo.version);
  lblBootPages.Caption  := 'BootPages: '+inttostr(BootInfo.Bootpages);
  if BootInfo.Crypt then begin
    lblCrypt.Caption  := 'Expect crypted file: yes';
  end else begin
    lblCrypt.Caption  := 'Expect crypted file: no';
  end;
end;

procedure TForm1.OpenSerial;
begin
  if BootLoader.GetStatus=bsIDLE then
    Memo1.Clear;
  if BootLoader.GetTTY <> cmbInterface.Text then
    BootLoader.SetTTY(cmbInterface.Text);
  if BootLoader.GetBaud <> strToInt(cmbBaud.Text) then
    BootLoader.SetBaud(strToInt(cmbBaud.Text));
  if cmbInterface.Items.IndexOf(cmbInterface.Text)=-1 then
    cmbInterface.Items.Add(cmbInterface.Text);
end;

function TForm1.CheckSerial: boolean;
begin
  result := IsTTYOK(cmbInterface.Text);
end;

procedure TForm1.SetButtons(enabled_: boolean);
begin
  btnConnect.Enabled := enabled_;
  btnProgram.Enabled := enabled_;
  btnDisconnect.Enabled:= enabled_;
end;

procedure TForm1.cmbInterfaceChange(Sender: TObject);
begin
  btnDisconnect.Click;
  SetButtons(CheckSerial);
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  i:integer;
  s:string;
begin
    Configfile := TIniFile.Create(ExtractFilePath(Application.ExeName)+'Bootloader.conf');
    FileList := TStringList.Create;
    i := 0;
    cmbFiles.Clear;
    FileList.clear;
    while Configfile.SectionExists('File'+inttostr(i)) do begin
      s := Configfile.ReadString('File'+inttostr(i),'FileName','');
      if FileExists(s)then begin
        cmbFiles.Items.add(ExtractFileName(s));
        FileList.add(s);
      end;
      inc(i);
    end;
    cmbFiles.ItemIndex:= Configfile.ReadInteger('GUI','FileIndex',0);
    cmbFiles.items.add('Browse..');
    i := 0;
    while Configfile.ValueExists('serials','serial'+inttostr(i)) do begin
      s := Configfile.ReadString('serials','serial'+inttostr(i),'');
      cmbInterface.Items.add(s);
      inc(i);
    end;
    cmbInterface.ItemIndex:= Configfile.ReadInteger('serial','SerialIndex',0);
    cmbBaud.ItemIndex:= Configfile.ReadInteger('serial','Baud',0);
    SetButtons(CheckSerial);

    BootLoader:=TBootLoader.create;
    BootLoader.OnStatus:=@OnBootStatus;
    BootLoader.OnRxByte:=@OnBootloaderRXByte;
    BootLoader.OnBootInfo := @OnBootInfo;
    BootLoader.OnError:= @OnBootError;
  { ser:=TBlockserial.Create;
   ser.OnStatus:= @OnStatus_;
   ser.LinuxLock:=false;
   ser.Connect('/dev/ttyUSB1');
   ser.Config(19200,8,'N',0,false,false); }
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: boolean);
var
  i:integer;
begin
  for i := 0 to FileList.Count-1 do begin
    Configfile.WriteString('File'+inttostr(i),'FileName',FileList[i]);
  end;
  Configfile.WriteInteger('GUI','FileIndex',cmbFiles.ItemIndex);

  for i := 0 to cmbInterface.Items.Count-1 do begin
    Configfile.WriteString('serials','serial'+inttostr(i),cmbInterface.Items[i]);
  end;
  Configfile.WriteInteger('serial','SerialIndex' ,cmbInterface.ItemIndex);
  Configfile.WriteInteger('serial','Baud' ,cmbBaud.ItemIndex);
  BootLoader.free;
end;

procedure TForm1.OnBootStatus(StatusMsg: string; Status: TBootloaderStatus; const ApplyNewLine:boolean);
begin
  if StatusMsg <> '' then
    Memo1.Lines.add(StatusMsg);
end;



procedure TForm1.OnBootloaderRXByte(RxByte: Byte);
begin
  if IsASCII(RxByte) then
    edtRxWin.Text := edtRxWin.Text + ' #' +inttohex(RxByte,2) +' '
  else
    edtRxWin.Text := edtRxWin.Text + '"'+char (RxByte)+'"';
end;

procedure TForm1.OnBootError(StatusMsg: string; Error: TBootloaderError);
begin
  if length(Memo1.Lines[Memo1.Lines.Count-1]) < 19 then begin
    Memo1.Lines[Memo1.Lines.Count-1] := Memo1.Lines[Memo1.Lines.Count-1] + #9+#9+#9+#9+#9+#9+StatusMsg;
  end else if length(Memo1.Lines[Memo1.Lines.Count-1]) < 28 then begin
    Memo1.Lines[Memo1.Lines.Count-1] := Memo1.Lines[Memo1.Lines.Count-1] + #9+#9+#9+#9+StatusMsg;
  end else if length(Memo1.Lines[Memo1.Lines.Count-1]) < 36 then begin
    Memo1.Lines[Memo1.Lines.Count-1] := Memo1.Lines[Memo1.Lines.Count-1] + #9+#9+StatusMsg;
  end else begin
    Memo1.Lines[Memo1.Lines.Count-1] := Memo1.Lines[Memo1.Lines.Count-1] + #9+StatusMsg;
  end;
end;

initialization
  {$I unit1.lrs}

end.

